######################################################################
### Drone Shield Personal Mesh Extractor Author:  Resin_Smoker 15FEB2010 
### Special Thanks: Exavier Macbeth
######################################################################
do
    function ExtractCustomShieldMeshBlueprint(bp)
        
        ### Verifiy the units Mesh and Mesh ID
        local meshid = bp.Display.MeshBlueprint
        if not meshid then return end
        local meshbp = original_blueprints.Mesh[meshid]
        if not meshbp then return end 
        
        ### Unit types the shield drone is not allowed to make shield meshes for
        local excludedCats = { 
            ### Command unit restrictions
            'SUBCOMMANDER','COMMAND', 
            ### Flying unit restrictions
            'HIGHALTAIR','GROUNDATTACK','BOMBER',
            ### Sub unit restrictions
            'T1SUBMARINE','T2SUBMARINE','NUKESUB',        
            ### Custom unit restrictions
            'HOLOGRAM','DRONE','MINE', 
            ### Misc unit restrictions
            'POD','SATELLITE','UNTARGETABLE',
            'SHIELD','WALL','NUKE','PROJECTILE', 
            'OPERATION','CIVILIAN','INSIGNIFICANTUNIT', 
            'UNSELECTABLE','BENIGN',                  
        }             
        
        ### Verify the unit type is one that we are allowed to enhance      
        for k, v in excludedCats do 
            if table.find(bp.Categories, v) then 
                #LOG('*** This Unit Type Not allowed: ', meshid)
                #LOG('*** Reason: ', excludedCats[k] )
                return       
            end           
        end 
                    
        ### Set a personal shield mesh for the unit based upon that units faction         	
        local pshieldmeshbp = table.deepcopy(meshbp)
        if pshieldmeshbp.LODs then
            for i,lod in pshieldmeshbp.LODs do      
                if table.find(bp.Categories, 'AEON') then 
                    #LOG('### AEON mesh created for UnitID: ', meshid)
                    lod.ShaderName = 'PhaseShield'                     
                    lod.LookupName = '/effects/entities/ShieldSection01/ShieldSection01b_Albedo.dds'                  
                elseif table.find(bp.Categories, 'CYBRAN') then 
                    #LOG('### CYBRAN mesh created for UnitID: ', meshid) 
                    lod.ShaderName = 'PhaseShield'                                                      
                    lod.LookupName = '/effects/entities/ForceField01/ForceField01_Albedo.dds'                                       
                elseif table.find(bp.Categories, 'UEF') then 
                    #LOG('### UEF mesh created for UnitID: ', meshid)  
                    lod.ShaderName = 'PhaseShield'                                       
                    lod.LookupName = '/effects/entities/CybranPhalanxSphere01/CybranPhalanxSphere01_Normals.dds' 
                elseif table.find(bp.Categories, 'SERAPHIM') then 
                    #LOG('### SERAPHIM mesh created for UnitID: ', meshid)   
                    lod.ShaderName = 'SeraphimPersonalShield'
                    lod.LookupName = '/effects/entities/ShieldSection01/ShieldSection01b_Albedo.dds'                  
                    lod.SecondaryName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'                                                                                            
                else    
                    LOG('### GENERIC mesh created for UnitID: ', meshid)                 
                    lod.ShaderName = 'PhaseShield'                                   
                    lod.LookupName = '/effects/entities/PhaseShield/PhaseShieldLookup.dds'                                               
                end
                ### Verify the end result of mesh via the log but comment out if not needed
                #if pshieldmeshbp.LODs then          
                #    for k, v in pshieldmeshbp.LODs do 
                #        LOG(k .. " = " ..  repr(v)) 
                #    end        
                #end                 
            end
        end
        pshieldmeshbp.BlueprintId = meshid .. '_PhaseShield'
        bp.Display.PShieldMeshBlueprint = pshieldmeshbp.BlueprintId
        MeshBlueprint(pshieldmeshbp)                  
    end		

	local OldModBlueprints = ModBlueprints
	function ModBlueprints(all_blueprints)
		OldModBlueprints(all_blueprints)
		for id,bp in all_blueprints.Unit do
			ExtractCustomShieldMeshBlueprint(bp)
		end
	end
end